@@ -10,21 +10,22 @@ with open('README.rst', 'r') as rmf:
     README = rmf.read()
 
 setup(
-    name = "mhci-netmhcpan-4.0-percentile-data",
+    name = "mhci-netmhcpan-4.1-ba-percentile-data",
     version = "1.0.0",
     author = "Jason Yan",
     author_email = "jyan@lji.org",
-    packages = ['mhci_netmhcpan_4_0_percentile_data'],
+    packages = ['mhci_netmhcpan_4_1_ba_percentile_data'],
     package_data = {
-        'mhci_netmhcpan_4_0_percentile_data': [
-            'netmhcpan_percentile_distribution_2018-01-15.p'
+        'mhci_netmhcpan_4_1_ba_percentile_data': [
+            'netmhcpan_ba_percentile_distribution_2020-08-12.p'
         ],
     },
-    description = 'Percentile data for mhci NetMHCPanPredictor 4.0',
+    test_suite = 'tests',
+    description = 'Percentile data for mhci NetMHCPanPredictor 4.1 BA',
     long_description = README,
     classifiers = [
         'Intended Audience :: Developers',
         'Operating System :: POSIX :: Linux',
-        'Programming Language :: Python :: 2',
+        'Programming Language :: Python :: 3',
     ],
 )
