@@ -12,7 +12,7 @@ use Sys::Hostname;
 my $version = "1.1";
 
 # variables used to save options
-my ($method, $rdir, $NetMHC, $NetMHCpan, $PickPocket, $n_pep, $n_bind, $thr_aff_S, $thr_aff_W, $thr_rank_S, $thr_rank_W, $dist, $inptype, $length, $tdir, $alleles, $a1, $a2, $a3, $a4, $training_MHC, $training_Pan_Pick, $file, $dirty, $hlaseq, $sort, $xls, $xlsfile, $filter, $rank_f, $aff_f, $w, $help);
+my ($method, $rdir, $NetMHC, $NetMHCpan, $PickPocket, $n_pep, $n_bind, $thr_aff_S, $thr_aff_W, $thr_rank_S, $thr_rank_W, $dist, $inptype, $length, $tdir, $alleles, $a1, $a2, $a3, $a4, $training_MHC, $training_Pan_Pick, $file, $dirty, $hlaseq, $sort, $xls, $xlsfile, $filter, $rank_f, $aff_f, $w, $help, $v);
 
 # associating option values with variables
 
@@ -49,6 +49,7 @@ my ($method, $rdir, $NetMHC, $NetMHCpan, $PickPocket, $n_pep, $n_bind, $thr_aff_
 'xls' => \$xls,
 'w' => \$w,
 'dirty' => \$dirty,
+'v' => \$v,
 'h' => \$help
 );
 
@@ -589,11 +590,13 @@ if ($hlaseq_option ne "") {
 
 	# check if pseudosequence contains gaps
 	if ($pseudosequence =~ m/-/) {
-		die "\nError. MHC input sequence incomplete. Pseudo sequence $pseudosequence contains gaps\n";
+		print "\nError. MHC input sequence incomplete. Pseudo sequence $pseudosequence contains gaps\n";
+		exit;
 	}
 	#check if pseudosequence is composed of letters only
 	if ($pseudosequence !~ m/^\w+$/) {
-		die "\nError. The error occurred obtaining pseudosequence from MHC sequence\n";
+		print "\nError. The error occurred obtaining pseudosequence from MHC sequence\n";
+		exit;
 	}	
 	# Getting the allele from the pseudosequence
 	my $allele = "";
@@ -1907,6 +1910,7 @@ sub runMHC {
 	#sleep(10);
 	#open (IN, "<", $tmpfile) or die "Can't open the file with NetMHC output $tmpfile $!\n";
 	
+	
 	open(IN, '-|', "$MHC $inptype -a $allele -l $length $file");
 	while (defined (my $line = <IN>)) {
 		chomp $line;
