@@ -3,7 +3,7 @@
 #import sys
 import os
 
-from util import get_standard_mhc_name
+from .util import get_standard_mhc_name
 
 def get_method_list():
     method_list = ['ann','smm','arb', 'comblib_sidney2008']
@@ -24,16 +24,16 @@ def write_mhc_list():
     dic = get_dic_model_list()
     method_list = get_method_list()
     for method in method_list:
-        print 'A list of available (MHC, PeptideLength) for ', method
-        print 'MHC\tPeptideLength'
+        print ('A list of available (MHC, PeptideLength) for ', method)
+        print ('MHC\tPeptideLength')
         model_list = dic[method]
         for model_name in model_list:
             (mhc, peptide_length) = get_standard_mhc_name(model_name)
             #temp = model_name.strip().split('-')
             #peptide_length = temp[-1]
             #mhc = '-'.join(temp[0:-1])
-            print mhc,'\t', peptide_length
-        print '\n'
+            print (mhc,'\t', peptide_length)
+        print ('\n')
 
 
 if __name__ == '__main__':
