@@ -4,21 +4,17 @@ import os
 import sys
 import math
 import bisect
-from util import * #@UnusedWildImport
+import pickle
+import re
+import logging
 
-from netmhc_4_0_executable import predict as predict_netmhc
-from netmhcpan_4_0_executable import predict_many as predict_netmhcpan
-from pickpocket_1_1_executable import predict_single as predict_pickpocket
-from netmhccons_1_1_executable import predict_sequence as predict_netmhccons
-from netmhcstabpan_1_0_executable import predict as predict_netmhcstabpan
+from util import InputError, UnexpectedInputError, PredictorError, FASTASequenceInput, get_species, InputData, get_peptides, MHCIAlleleData, MethodSet, get_mhc_list, get_path_model, median, PredictorSelectionB
+from setupinfo import SetupInfo
 
-from mhci_netmhcpan_4_0_percentile_data import score_distributions as netmhcpan_4_score_distributions
-from mhci_ann_predictor_percentile_data import score_distributions as netmhc_4_score_distributions
-from mhci_netmhcstabpan_predictor_percentile_data import score_distributions as netmhcstabpan_score_distributions
 from PercentilesCalculators import MHCIPercentilesCalculator
 
 from iedbtools_utilities.sequence_io import FASTASequenceInput
-from allele_info import MHCIAlleleData
+from allele_info import MHCIAlleleData, is_user_defined_allele
 
 class PredictorSet(object):
     """A set of predictors. An appropriate set of predictors should be loaded for each version."""
@@ -38,7 +34,8 @@ class PredictorSet(object):
 
         dic = {}
         for method in method_list:
-            dic[method] = self.get_predictor(method, path_method, path_data)                      # Experimental
+            # JY: removed  path_method, path_data from the parameters
+            dic[method] = self.get_predictor(method)                      # Experimental
         return dic
 
     def get_predictor(self, method, method_used=None):
@@ -51,6 +48,7 @@ class PredictorSet(object):
             'smmpmbec':          SMMMatrix(path_method, path_data, method_name='smmpmbec', method_used=method_used),          # Must be included.
             'arb':               ARBMatrix(path_method, path_data),                                  # Must be included.
             'netmhcpan':         NetMHCpanPredictor(path_method, path_data, method_used),                         # Experimental
+            'netmhcpan_el':      NetMHCpanELPredictor(path_method, path_data, method_used),                         # Experimental
             'comblib_sidney2008':CombinatorialLibrary(path_method, path_data, 'comblib_sidney2008'), # Must be included.
             'comblib_udaka2000': CombinatorialLibrary(path_method, path_data, 'comblib_udaka2000'),  # Must be included.
             'consensus':         ConsensusPredictor(self.setupinfo),                                 # Experimental
@@ -60,16 +58,18 @@ class PredictorSet(object):
         }[method]
         return predictor
 
+    # JY: unnecessary methods?
     def get_method_name_list(self):
-        key_list = self.dic_predictor.keys()
+        key_list = self.get_dic_predictor().keys()
         key_list.sort()
         return key_list
 
+    # JY: unnecessary methods?
     def get_predictor_list(self, method_list):
         predictor_list = []
         for method in method_list:
-            if (self.dic_predictor.has_key(method) == True):
-                predictor = self.dic_predictor[method]
+            if method in self.get_dic_predictor():
+                predictor = self.get_dic_predictor()[method]
                 predictor_list.append(predictor)
         return predictor_list
 
@@ -135,7 +135,7 @@ class ARBMatrix(object):
             for e in entries[1:]:
                 numbers.append(math.log10(float(e)))
             if len(numbers)!=self.length:
-                raise PredictorError("Invalid number of columns in ARB matrix: " + str(len(numbers)), " expected: " + str(self.length) + ".")
+                raise PredictorError("Invalid number of columns in ARB matrix: " + str(len(numbers)) + " expected: " + str(self.length) + ".")
             self.mat[line[0]]=tuple(numbers)
         p = infile.find("SLOPE")
         self.slope = float(infile[p + 5:infile.find("\n",p+1)])
@@ -144,18 +144,18 @@ class ARBMatrix(object):
 
     def pickle_dump(self, file_name):
         fout=open(file_name,"wb")
-        cPickle.dump(self.length, fout)
-        cPickle.dump(self.mat,fout)
-        cPickle.dump(self.slope,fout)
-        cPickle.dump(self.intercept,fout)
+        pickle.dump(self.length, fout)
+        pickle.dump(self.mat,fout)
+        pickle.dump(self.slope,fout)
+        pickle.dump(self.intercept,fout)
         fout.close()
 
     def pickle_load(self, file_name):
         fin = open(file_name,"rb")
-        self.length = cPickle.load(fin)
-        self.mat = cPickle.load(fin)
-        self.slope = cPickle.load(fin)
-        self.intercept = cPickle.load(fin)
+        self.length = pickle.load(fin)
+        self.mat = pickle.load(fin)
+        self.slope = pickle.load(fin)
+        self.intercept = pickle.load(fin)
         fin.close()
 
 
@@ -214,10 +214,12 @@ class SMMMatrix:
             for e in entries[1:]:
                 numbers.append(float(e))
             if len(numbers)!=self.length:
-                raise PredictorError("Invalid number of columns in SMM matrix: " + str(len(numbers)), " expected: " + str(self.length) + ".")
+                raise PredictorError("Invalid number of columns in SMM matrix: " + str(len(numbers)) + " expected: " + str(self.length) + ".")
             self.mat[line[0]]=tuple(numbers)
         self.offset = float(lines[21])
 
+    '''
+    # unnecessary methods?
     def save_text_file(self, outfile):
         outfile.write("NumCols:\t" + str(self.length) +"\n")
         for letter in sorted(self.keys()):
@@ -226,19 +228,20 @@ class SMMMatrix:
                 outfile.write("\t" + str(val))
             outfile.write("\n")
         outfile.write(str(self.offset))
+    '''
 
     def pickle_dump(self, file_name):
         fout = open(file_name,"wb")
-        cPickle.dump(self.length, fout)
-        cPickle.dump(self.mat,fout)
-        cPickle.dump(self.offset,fout)
+        pickle.dump(self.length, fout)
+        pickle.dump(self.mat,fout)
+        pickle.dump(self.offset,fout)
         fout.close()
 
     def pickle_load(self, file_name):
         fin = open(file_name,"rb")
-        self.length = cPickle.load(fin)
-        self.mat = cPickle.load(fin)
-        self.offset = cPickle.load(fin)
+        self.length = pickle.load(fin)
+        self.mat = pickle.load(fin)
+        self.offset = pickle.load(fin)
         fin.close()
         
 
@@ -256,16 +259,13 @@ class ANNPredictor:
         return 'IC50 (nM)'
 
     def predict_sequence(self,sequence,pred):
+        from netmhc_4_0_executable import predict as predict_netmhc
+
         """ Given one protein sequence, break it up into peptides, return their predicted binding scores. """
         
         # Eliminate the asterisk (*) and colon (:) from the allele_name
         allele_name = self.mhc.replace('*', '').replace(':','')
         scores = predict_netmhc(allele_name, str(self.length), sequence)
-        
-        # get percentile scores
-        args = ('ann', self.mhc.replace("*",""), self.length)
-        ps = PercentileScore(os.path.dirname(self.path_data), 'consensus', args)
-        percentile = ps.get_percentile_score(scores)
 
         # get percentile scores
         allele_length_pair = (self.mhc.replace("*",""), self.length)
@@ -275,14 +275,13 @@ class ANNPredictor:
     def get_percentiles_for_scores(self, raw_scores, allele_length_pair):
         ''' Returns the percentile scores for the raw scores passed. 
         '''
+        from mhci_ann_predictor_percentile_data import score_distributions as netmhc_4_score_distributions
         percentiles_calculator = MHCIPercentilesCalculator(netmhc_4_score_distributions)
         allele, binding_length = allele_length_pair
         percentiles = percentiles_calculator.get_percentile_scores(
                         raw_scores, 'ann', allele, binding_length)
         return percentiles
 
-
-
 class NetMHCpanPredictor:
     def __init__(self, path_method, path_data, method_used=None):
         self.path_data = os.path.join(path_data)
@@ -295,6 +294,7 @@ class NetMHCpanPredictor:
 
     def predict_sequence(self, sequence, pred):
         """ Given one protein sequence, break it up into peptides, return their predicted binding scores. """
+        from netmhcpan_4_1_executable import predict_many as predict_netmhcpan
         if self.mhc != "User defined":
             allele_name_or_sequence = self.mhc
         else:
@@ -304,24 +304,94 @@ class NetMHCpanPredictor:
         input_sequence_list = sequence.split()
         results = predict_netmhcpan(input_sequence_list, [(allele_name_or_sequence, self.length)])
 
+        score_list = []
+        core_list = []
+        icore_list = []
         for scores in results.values():
+            for ind in range(0, len(scores)-3, 3) :
+                score_list.append( scores[ind] )
+                core_list.append( scores[ind + 1])
+                icore_list.append( scores[ind + 2])
+            # Have to add the very last set
+            score_list.append( scores[-3] )
+            core_list.append( scores[-2] )
+            icore_list.append( scores[-1] )
+
             if self.mhc != "User defined":
                 allele_length_pair = (allele_name_or_sequence, self.length)
-                percentile = self.get_percentiles_for_scores(scores, allele_length_pair)
-                return zip(scores, percentile)
+                percentile = self.get_percentiles_for_scores(score_list, allele_length_pair)
+                return zip(core_list, icore_list, score_list, percentile)
             else:
-                return scores
+                return zip(core_list, icore_list, score_list)
+
     def get_percentiles_for_scores(self, raw_scores, allele_length_pair):
         ''' Returns the percentile scores for the raw scores passed. 
         '''
+        from mhci_netmhcpan_4_1_ba_percentile_data import score_distributions as netmhcpan_4_score_distributions
         percentiles_calculator = MHCIPercentilesCalculator(netmhcpan_4_score_distributions)
         allele, binding_length = allele_length_pair
         try:
             percentiles = percentiles_calculator.get_percentile_scores(
-                            raw_scores, 'netmhcpan', allele, binding_length)
+                            raw_scores, 'netmhcpan_ba', allele, binding_length)
         except ValueError:
             if is_user_defined_allele(allele_length_pair.allele):
-                percentiles = [None for i in xrange(len(raw_scores))]
+                percentiles = [None for i in range(len(raw_scores))]
+            else:
+                raise
+        return percentiles
+
+class NetMHCpanELPredictor:
+    def __init__(self, path_method, path_data, method_used=None):
+        self.path_data = os.path.join(path_data)
+        self.method = method_used
+
+    def initialize(self, mhc, length, hla_seq=None):
+        self.mhc = mhc
+        self.length = length
+        self.hla_seq = hla_seq
+
+    def predict_sequence(self, sequence, pred):
+        """ Given one protein sequence, break it up into peptides, return their predicted binding scores. """
+        from netmhcpan_4_1_executable import predict_many as predict_netmhcpan
+        if self.mhc != "User defined":
+            allele_name_or_sequence = self.mhc
+        else:
+            si = FASTASequenceInput(self.hla_seq)
+            allele_name_or_sequence = "".join(si.as_amino_acid_text())
+        
+        input_sequence_list = sequence.split()
+        results = predict_netmhcpan(input_sequence_list, [(allele_name_or_sequence, self.length)], el=True)
+        score_list = []
+        core_list = []
+        icore_list = []
+        for scores in results.values():
+            for ind in range(0, len(scores)-3, 3) :
+                score_list.append( scores[ind] )
+                core_list.append( scores[ind + 1])
+                icore_list.append( scores[ind + 2])
+            # Have to add the very last set
+            score_list.append( scores[-3] )
+            core_list.append( scores[-2] )
+            icore_list.append( scores[-1] )
+            if self.mhc != "User defined":
+                allele_length_pair = (allele_name_or_sequence, self.length)
+                percentile = self.get_percentiles_for_scores(score_list, allele_length_pair)
+                return zip(core_list, icore_list, score_list, percentile)
+            else:
+                return zip(core_list, icore_list, score_list)
+
+    def get_percentiles_for_scores(self, raw_scores, allele_length_pair):
+        ''' Returns the percentile scores for the raw scores passed. 
+        '''
+        from mhci_netmhcpan_4_1_el_percentile_data import score_distributions as netmhcpan_4_el_score_distributions
+        percentiles_calculator = MHCIPercentilesCalculator(netmhcpan_4_el_score_distributions)
+        allele, binding_length = allele_length_pair
+        try:
+            percentiles = percentiles_calculator.get_percentile_scores(
+                            raw_scores, 'netmhcpan_el', allele, binding_length)
+        except ValueError:
+            if is_user_defined_allele(allele_length_pair.allele):
+                percentiles = [None for i in range(len(raw_scores))]
             else:
                 raise
         return percentiles
@@ -338,6 +408,7 @@ class PickPocketPredictor:
 
     def predict_sequence(self, sequence, pred):
         """ Given one protein sequence, break it up into peptides, return their predicted binding scores. """
+        from pickpocket_1_1_executable import predict_single as predict_pickpocket
         if self.mhc != "User defined":
             allele_name_or_sequence = self.mhc
         else:
@@ -366,10 +437,10 @@ class NetMHCStabPanPredictor:
 
     def predict_sequence(self,sequence,pred):
         """ Given one protein sequence, break it up into peptides, return their predicted binding scores. """
+        from netmhcstabpan_1_0_executable import predict as predict_netmhcstabpan
         scores = predict_netmhcstabpan(self.mhc, str(self.length), sequence)
 
         # get percentile scores
-
         allele_length_pair = (self.mhc, self.length)
         percentile = self.get_percentiles_for_scores(scores, allele_length_pair)
         return zip(scores, percentile)
@@ -378,6 +449,7 @@ class NetMHCStabPanPredictor:
     def get_percentiles_for_scores(self, raw_scores, allele_length_pair):
         ''' Returns the percentile scores for the raw scores passed. 
         '''
+        from mhci_netmhcstabpan_predictor_percentile_data import score_distributions as netmhcstabpan_score_distributions
         percentiles_calculator = MHCIPercentilesCalculator(netmhcstabpan_score_distributions)
         allele, binding_length = allele_length_pair
         percentiles = percentiles_calculator.get_percentile_scores(
@@ -397,6 +469,7 @@ class NetMHCconsPredictor:
 
     def predict_sequence(self, sequence, pred):
         """ Given one protein sequence, break it up into peptides, return their predicted binding scores. """
+        from netmhccons_1_1_executable import predict_sequence as predict_netmhccons
         if self.mhc != "User defined":
             allele_name_or_sequence = self.mhc
         else:
@@ -406,19 +479,20 @@ class NetMHCconsPredictor:
         scores = predict_netmhccons(sequence, (allele_name_or_sequence, self.length))
 
         # get percentile scores
-        args = ('netmhccons', self.mhc, self.length)
-        ps = PercentileScore(os.path.dirname(self.path_data), 'netmhccons', args)
-        percentile = ps.get_percentile_score(scores)
+        allele_length_pair = (self.mhc, self.length)
+        percentile = self.get_percentiles_for_scores(scores, allele_length_pair)
+
         return zip(scores, percentile)
 
-        # if self.mhc != "User defined":
-        #     args = ("netmhccons", self.mhc, int(self.length))
-        #     ps = PercentileScore(self.path_data, "netmhccons", args)
-        #     percentile = ps.get_percentile_score(results)
-        #     return zip(results, percentile)
-        # else:
-        #     return results
-        
+    def get_percentiles_for_scores(self, raw_scores, allele_length_pair):
+        ''' Returns the percentile scores for the raw scores passed. 
+        '''
+        from mhci_netmhccons_percentile_data  import score_distributions as netmhccons_score_distributions
+        percentiles_calculator = MHCIPercentilesCalculator(netmhccons_score_distributions)
+        allele, binding_length = allele_length_pair
+        percentiles = percentiles_calculator.get_percentile_scores(
+                        raw_scores, 'netmhccons', allele, binding_length)
+        return percentiles        
     
 class CombinatorialLibrary:
     """Can load and save SMM matrices in multiple formats and use them to score sequences """
@@ -458,7 +532,7 @@ class CombinatorialLibrary:
         args = ('comblib_sidney2008', self.mhc.replace("*",""), self.length)
         ps = PercentileScore(self.path_data, 'consensus', args)
         percentile = ps.get_percentile_score(scores)
-        return zip(scores, percentile)
+        return list(zip(scores, percentile))
 
     def predict_peptide_list(self, peptide_list):
         scores = []
@@ -475,8 +549,8 @@ class CombinatorialLibrary:
         return (tuple(scores))
 
     def read_data_cpickle(self,fname):
-        f = open(fname,'r')
-        data = cPickle.load(f)
+        f = open(fname,'rb')
+        data = pickle.load(f)
         f.close()
         return data
 
@@ -519,16 +593,16 @@ class CombinatorialLibrary:
 
     def pickle_dump(self, file_name):
         fout = open(file_name,"wb")
-        cPickle.dump(self.length, fout)
-        cPickle.dump(self.mat,fout)
-        cPickle.dump(self.offset,fout)
+        pickle.dump(self.length, fout)
+        pickle.dump(self.mat,fout)
+        pickle.dump(self.offset,fout)
         fout.close()
 
     def pickle_load(self, file_name):
         fin = open(file_name,"rb")
-        self.length = cPickle.load(fin)
-        self.mat = cPickle.load(fin)
-        self.offset = cPickle.load(fin)
+        self.length = pickle.load(fin)
+        self.mat = pickle.load(fin)
+        self.offset = pickle.load(fin)
         fin.close()
 
 
@@ -568,8 +642,8 @@ class ConsensusPredictor(object):
 
     def read_score_distributions(self):
         fname = os.path.join(self.path_data, 'consensus', 'distribution_consensus_bin.cpickle')
-        f = open(fname, 'r')
-        dic = cPickle.load(f)
+        f = open(fname, 'rb')
+        dic = pickle.load(f)
         f.close()
         return dic
 
@@ -590,9 +664,12 @@ class ConsensusPredictor(object):
              
             if method_name == 'smm' or method_name == 'ann' or method_name == 'comblib_sidney2008':
                 spercentile = predictor.predict_sequence(sequence,pred)
+                spercentile = list(spercentile)
+                logging.debug('spercentile:%s' % str(spercentile))
                 scores = tuple([sp[0] for sp in spercentile])
                 percentile = tuple([sp[1] for sp in spercentile])
                 ic50scores.append(scores)
+                logging.debug('percentile:%s' % str(percentile))
                 scores_predictor.append(percentile)
             else:
                 scores = predictor.predict_sequence(sequence,pred)
@@ -610,7 +687,8 @@ class ConsensusPredictor(object):
         if pred == 'submit_processing':
             return tuple(scores_consensus)
         else:
-            ic50_ranks = zip(ic50scores, scores_predictor)
+            logging.debug('pred != submit_processing')
+            ic50_ranks = list(zip(ic50scores, scores_predictor))
             return tuple(scores_consensus), ic50_ranks    #ic50scores
         
     def search(self, a, x):
@@ -651,6 +729,7 @@ class ConsensusPredictor(object):
         dic['smm']                = SMMMatrix(path_method, path_data) # Must be included.
         dic['arb']                = ARBMatrix(path_method, path_data) # Must be included.
         dic['netmhcpan']          = NetMHCpanPredictor(path_method, path_data) # Experimental
+        dic['netmhcpan_el']          = NetMHCpanELPredictor(path_method, path_data) # Experimental
         dic['comblib_sidney2008'] = CombinatorialLibrary(path_method, path_data, 'comblib_sidney2008')
         dic['comblib_udaka2000']  = CombinatorialLibrary(path_method, path_data, 'comblib_udaka2000')
         #dic['consensus']          = ConsensusPredictor(path_method, path_data) # Experimental
@@ -707,8 +786,8 @@ class PercentileScore:
 
     def read_score_distributions(self):
         fname = os.path.join(self.path_data, self.method, 'distribution_{0}_bin.cpickle'.format(self.method))
-        f = open(fname, 'r')
-        dic = cPickle.load(f)
+        f = open(fname, 'rb')
+        dic = pickle.load(f)
         f.close()
         return dic
     
@@ -751,6 +830,8 @@ class MHCBindingPredictions:
         prediction_score_unit = 'ic50'
         if (self.method=='comblib_sidney2008'):
             prediction_score_unit = 'score'
+        elif (self.method=='netmhcpan_el'):
+            prediction_score_unit = 'score'
         elif self.method=='consensus':
             prediction_score_unit = 'consensus_percentile_rank'
         elif self.method=='IEDB_recommended':
@@ -766,24 +847,25 @@ class MHCBindingPredictions:
             num_methods = len(method_set_selected)
             if num_methods >= 2: method_set_selected.append('consensus')
             method_set_selected.sort()
-        else: method_set_selected.append(method)
+        else: 
+            method_set_selected.append(method)
         return method_set_selected
 
     def predict(self, sequence_list, pred=None):
         results = []
-        
+
         if self.method == 'IEDB_recommended':
             for allele, length in self.tool_selection:
                 method_name = ','.join(self.method_lookup(allele, length))
                 # if 'recommended' is passed from MHC-I processing, overwrite the method_name to 'netmhcpan'
                 if pred == 'submit_processing':
-                    method_name = 'netmhcpan'
+                    method_name = 'netmhcpan_el'
                      
                 predictor = ''
-                if method_name != 'netmhcpan':
-                    predictor = self.predictor_set.get_predictor('consensus', 'IEDB_recommended')
+                if method_name in ['netmhcpan', 'netmhcpan_el',]:
+                    predictor = self.predictor_set.get_predictor('netmhcpan_el', 'IEDB_recommended')
                 else:
-                    predictor = self.predictor_set.get_predictor('netmhcpan', 'IEDB_recommended')
+                    predictor = self.predictor_set.get_predictor('consensus', 'IEDB_recommended')
                 
                 # run the 'initialize' method for a specific predictor class
                 predictor.initialize(allele, int(length))
@@ -795,7 +877,6 @@ class MHCBindingPredictions:
         else:
             #if 'IEDB recommended' is not selected, only one method will be chosen:
             predictor = self.predictor_set.get_predictor(self.method)
-            
             self.method_set_selected = [self.method]
             if ((self.hla_seq is None) | (self.hla_seq == "")):
                 """
@@ -806,14 +887,17 @@ class MHCBindingPredictions:
                 4. Collect scores as a list.
                 """
                 for allele, length in self.tool_selection:
-                    method_name = ','.join(self.method_lookup(allele, length))
+                    if self.method == 'consensus':
+                        method_name = ','.join(self.method_lookup(allele, length))
+                    else:
+                        method_name = self.method
                  
                     if not method_name:
                         method_name = 'netmhcpan'
- 
-                    if (self.method == 'netmhcpan'): predictor.initialize(allele, int(length), self.hla_seq)
-                    else: predictor.initialize(allele, int(length))
- 
+                    if (self.method == 'netmhcpan'): 
+                        predictor.initialize(allele, int(length), self.hla_seq)
+                    else: 
+                        predictor.initialize(allele, int(length))
                     scores = []
                     for sequence in sequence_list:
                         scores.append(predictor.predict_sequence(sequence,pred))
@@ -827,7 +911,7 @@ class MHCBindingPredictions:
                 mhc = 'User defined'
                 length = (self.length).pop(0)
                 predictor.initialize(mhc, int(length), self.hla_seq)
-                method_name = "netmhcpan"
+                method_name = "netmhcpan_el"
                 scores = []
                 for sequence in sequence_list:
                     scores.append(predictor.predict_sequence(sequence,pred))
@@ -840,6 +924,7 @@ class MHCBindingPredictions:
         consensus_methods = ['ann', 'smm', 'comblib_sidney2008', 'netmhcpan']
         miad = MHCIAlleleData()
         method_list = miad.get_method_names(allele_name=allele, binding_length=length)
+        # intersection of sets.
         method_used = list(set(consensus_methods) & set(method_list))
         if len(method_used) > 1 and 'netmhcpan' in method_list:
             method_used.remove('netmhcpan')
