@@ -1,6 +1,6 @@
 import os, sys; 
 MHCI_HOME_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir)); sys.path.append(MHCI_HOME_DIR)
-import cPickle
+import pickle
 import re
 
 from setupinfo import SetupInfo
@@ -141,7 +141,8 @@ class InputData(object):
 
     def set_peptide_length(self, length):
         self.length = length
-        
+    
+    # JY: unnecessary methods?
     def set_input_protein(self, seq_list):
         '''Assumes a list of strings.'''
         self.input_protein = None
@@ -149,7 +150,7 @@ class InputData(object):
         for seq in seq_list:
             temp_protein.add_protein(seq)
         self.input_protein = temp_protein
-
+    # JY: unnecessary methods?
     def set_input_protein_mhc(self, usermhc_seq):
         self.mhc = 'UserDefined'
         self.input_protein_mhc = None
@@ -177,8 +178,8 @@ class PredictorSelectionB(object):
         
          
         miad = MHCIAlleleData()
-        allele_list = miad.get_allele_names_for_method(method_name=pred_method.replace('IEDB_', ''))
-        binding_length_list = miad.get_allowed_peptide_lengths(method_name=pred_method.replace('IEDB_', ''), allele_name=allele_name)
+        allele_list = miad.get_allele_names_for_method(method_name=pred_method.replace('IEDB_', '').replace('netmhcpan_el','netmhcpan').replace('netmhcpan', 'netmhcpan-4.1'))
+        binding_length_list = miad.get_allowed_peptide_lengths(method_name=pred_method.replace('IEDB_', '').replace('netmhcpan_el','netmhcpan').replace('netmhcpan', 'netmhcpan-4.1'), allele_name=allele_name)
         species = miad.get_species_for_allele_name(allele_name=allele_name)
         
         if allele_name not in allele_list:
@@ -194,7 +195,7 @@ class PredictorSelectionB(object):
                         tool_selection.append((allele_name, int(binding_length)))
          
         if len(tool_selection) == 0:
-            raise(UnexpectedInputError("Could not find tools matching species='{}', allele='{}', length='{}', method='{}'." % (species, allele_name, length, pred_method)))
+            raise(UnexpectedInputError("Could not find tools matching species='%s', allele='%s', length='%s', method='%s'." % (species, allele_name, length, pred_method)))
         return self.clean(tool_selection)
  
     def clean(self, lst):
@@ -338,12 +339,13 @@ class MethodSet(object):
         dic['pickpocket']          = (9, 'PickPocket')
         dic['netmhccons']          = (10, 'NetMHCcons')
         dic['netmhcstabpan'] = (11, 'netMHCstabpan')
+        dic['netmhcpan_el'] = (12, 'NetMHCpan_EL')
         return dic
 
     def get_dic_method_list(self):
         '''The order of the methods is important.'''
         dic={}
-        dic['20130222'] = ['ann', 'comblib_sidney2008', 'consensus', 'IEDB_recommended', 'netmhcpan', 'smm', 'smmpmbec', 'pickpocket', 'netmhccons', 'netmhcstabpan', ]
+        dic['20130222'] = ['ann', 'comblib_sidney2008', 'consensus', 'IEDB_recommended', 'netmhcpan', 'smm', 'smmpmbec', 'pickpocket', 'netmhccons', 'netmhcstabpan', 'netmhcpan_el',]
         dic['20090901B'] = ['ann', 'arb', 'comblib_sidney2008', 'consensus', 'IEDB_recommended', 'netmhcpan', 'smm', 'smmpmbec']
         dic['20090901'] = ['ann', 'smm', 'arb', 'smmpmbec', 'comblib_sidney2008', 'consensus']
         dic['20071227'] = ['ann', 'smm', 'arb']
@@ -433,7 +435,8 @@ def get_standard_mhc_name_b(mhc):
     mhc = mhc.replace('HLA-C-','HLA C*')
     mhc = mhc.replace('HLA-E-','HLA E*')
     mhc = mhc.replace('HLA-G-','HLA G*')
-    mhc = mhc.replace('H-2-','H-2 ')
+    # return H-2-Db instead of H-2 Db
+    #mhc = mhc.replace('H-2-','H-2 ')
     mhc = mhc.replace('Mamu-A-','Mamu A*')
     mhc = mhc.replace('Mamu-B-','Mamu B*')
     mhc = mhc.replace('Patr-A-','Patr A*')
@@ -480,11 +483,11 @@ def median(v_temp):
     v.sort()
     value_median = None
     if (is_even(len(v)) == True):
-        index_a = len(v)/2 - 1
+        index_a = len(v)//2 - 1
         index_b = index_a + 1
         value_median = (v[index_a] + v[index_b])/2.0
     else:
-        index = len(v)/2
+        index = len(v)//2
         value_median = v[index]
     return value_median
 
@@ -501,7 +504,7 @@ def rankdata(v):
     dic={}
     for (index,x,rank) in v_rank:
         key = x
-        if (dic.has_key(key) == False):
+        if key not in dic:
             dic[key] = [rank]
         else:
             temp = dic[key]
@@ -509,12 +512,13 @@ def rankdata(v):
     rank_list = [mean(dic.get(x)) for (index,x,rank) in v_rank]  # This averages ranks if their corresponding values are the same.
     return rank_list
 
+# JY: unnecessary and outdated method?
 def read_available_model_list():
     '''For each method, a list of available (mhc,length) is provided.'''
     setupinfo = SetupInfo()
     fname =setupinfo.fname_tool_list
     f=open(fname,'r')
-    model_list = cPickle.load(f)
+    model_list = pickle.load(f)
     f.close()
     return model_list
 
@@ -528,7 +532,7 @@ def read_available_model_list_b(version='20130222'):
     # The following block of code should be done in the very beginning of initialization of tools.
     dic_model_list={}
     for method in method_list:
-        fname = os.path.join(setupinfo.path_data, method,'model_list.txt')
+        fname = os.path.join(setupinfo.path_data, method.replace('netmhcpan_el','netmhcpan'),'model_list.txt')
         f=open(fname,'r'); lines=f.readlines(); f.close()
         model_list = [line.split('\t')[0].replace('_','-').strip() for line in lines] # Assumes now 'HLA-A-0201-9 form.
         dic_model_list[method] = model_list
@@ -600,10 +604,10 @@ def list_mhc():
     tools_list = list(temp)
     tools_list.sort()
 
-    print "MHC", "\t", "Peptide Length"
+    print ("MHC", "\t", "Peptide Length")
     for row in tools_list:
         (species, mhc, peptidelength) = row
-        print mhc, "\t", peptidelength
+        print (mhc, "\t", peptidelength)
     return tools_list
 
 def shared_list_mhc():
@@ -616,12 +620,14 @@ def shared_list_mhc():
     cset = set(tools_list_c)
     shared_list_mhc = list(aset.intersection(bset).intersection(cset))
     shared_list_mhc.sort()
-    print "MHC", "\t", "Peptide Length"
+    print ("MHC", "\t", "Peptide Length")
     for row in shared_list_mhc:
         (species, mhc, peptidelength) = row
-        print mhc, "\t", peptidelength
+        print (mhc, "\t", peptidelength)
     return shared_list_mhc
 
+'''
+# JY: Outdated function
 def test_predictor_selection():
     setupinfo = SetupInfo() # assumes version 20090901B.
     path_data = setupinfo.path_data
@@ -633,7 +639,7 @@ def test_predictor_selection():
     method_list = ['ann','smm','comblib_sidney2008', 'netmhcpan']  # This is the predictors used to build 'consensus'
     psb = PredictorSelectionB(path_data, pred_method, species, mhc, length)
     model_list = psb.get_model_list_combined()
-    print 'debug model_list ', len(model_list)
+    print ('debug model_list ', len(model_list))
     content = []
     for name in model_list:
         method_list_available = psb.get_available_methods(name, method_list)
@@ -641,8 +647,8 @@ def test_predictor_selection():
         content.append(line)
     content.sort()
     for line in content:
-        print line
+        print (line)
 
 if __name__ == '__main__':
     test_predictor_selection()
-
+'''
